@@ -1,8 +1,8 @@
 /********************************************
 rexp.h
-copyright 2008-2010,2012, Thomas E. Dickey
+copyright 2008-2020,2021, Thomas E. Dickey
 copyright 2010, Jonathan Nieder
-copyright 1991, Michael D. Brennan
+copyright 1991,2014, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -12,41 +12,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: rexp.h,v 1.25 2012/06/27 09:08:10 tom Exp $
- * @Log: rexp.h,v @
- * Revision 1.2  1993/07/23  13:21:35  mike
- * cleanup rexp code
- *
- * Revision 1.1.1.1  1993/07/03  18:58:27  mike
- * move source to cvs
- *
- * Revision 3.6  1992/01/21  17:31:45  brennan
- * moved ison() macro out of rexp[23].c
- *
- * Revision 3.5  91/10/29  10:53:55  brennan
- * SIZE_T
- *
- * Revision 3.4  91/08/13  09:10:02  brennan
- * VERSION .9994
- *
- * Revision 3.3  91/06/15  09:40:25  brennan
- * gcc defines __STDC__ but might not have stdlib.h
- *
- * Revision 3.2  91/06/10  16:18:19  brennan
- * changes for V7
- *
- * Revision 3.1  91/06/07  10:33:18  brennan
- * VERSION 0.995
- *
- * Revision 1.3  91/06/05  08:57:57  brennan
- * removed RE_xmalloc()
- *
- * Revision 1.2  91/06/03  07:23:26  brennan
- * changed type of RE_error_trap
- *
- * Revision 1.1  91/06/03  07:05:41  brennan
- * Initial revision
- *
+ * $MawkId: rexp.h,v 1.39 2021/05/28 08:23:39 tom Exp $
  */
 
 #ifndef  REXP_H
@@ -55,7 +21,7 @@ the GNU General Public License, version 2, 1991.
 #include "nstd.h"
 #include "types.h"
 #include <stdio.h>
-#include  <setjmp.h>
+#include <setjmp.h>
 
 extern PTR RE_malloc(size_t);
 extern PTR RE_realloc(void *, size_t);
@@ -68,27 +34,30 @@ extern void RE_free(void *);
 
 /*  finite machine  state types  */
 
-#define  M_STR     	0	/* matching a literal string */
-#define  M_CLASS   	1	/* character class */
-#define  M_ANY     	2	/* arbitrary character (.) */
-#define  M_START   	3	/* start of string (^) */
-#define  M_END     	4	/* end of string ($) */
-#define  M_U       	5	/* arbitrary string (.*) */
-#define  M_1J      	6	/* mandatory jump */
-#define  M_2JA     	7	/* optional (undesirable) jump */
-#define  M_2JB     	8	/* optional (desirable) jump */
-#define  M_SAVE_POS	9	/* push position onto stack */
-#define  M_2JC     	10	/* pop pos'n, optional jump if advanced */
-#define  M_ACCEPT  	11	/* end of match */
-#define  U_ON      	12
+typedef enum {
+    M_STR			/* matching a literal string */
+    ,M_CLASS			/* character class */
+    ,M_ANY			/* arbitrary character (.) */
+    ,M_START			/* start of string (^) */
+    ,M_END			/* end of string ($) */
+    ,M_U			/* arbitrary string (.*) */
+    ,M_1J			/* mandatory jump */
+    ,M_2JA			/* optional (undesirable) jump */
+    ,M_2JB			/* optional (desirable) jump */
+    ,M_SAVE_POS			/* push position onto stack */
+    ,M_2JC			/* pop pos'n, optional jump if advanced */
+    ,M_ACCEPT			/* end of match */
+    ,U_ON			/* ...distinct from the preceding */
+} MAWK_REGEX;
 
 #define  U_OFF     0
 #define  END_OFF   0
-#define  END_ON    (2*U_ON)
+#define  END_ON    (2*U_ON)	/* ...distinct from the preceding */
+
+#define  L_CURL         '{'
+#define  R_CURL         '}'
 
 typedef UChar BV[32];		/* bit vector */
-typedef char SType;
-typedef UChar SLen;
 
 typedef struct {
     SType s_type;
@@ -98,6 +67,11 @@ typedef struct {
 	BV *bvp;		/*  class  */
 	int jump;
     } s_data;
+#ifndef NO_INTERVAL_EXPR
+    Int it_min;			/* used for s_type == M_2JC */
+    Int it_max;			/* used for s_type == M_2JC */
+    Int it_cnt;
+#endif
 } STATE;
 
 #define  STATESZ  (sizeof(STATE))
@@ -107,34 +81,43 @@ typedef struct {
 } MACHINE;
 
 /*  tokens   */
-#define  T_NONE   0		/* no token */
-#define  T_OR     1		/* | */
-#define  T_CAT    2		/* binary operator */
-#define  T_STAR   3		/* * */
-#define  T_PLUS   4		/* + */
-#define  T_Q      5		/* ? */
-#define  T_LP     6		/* ( */
-#define  T_RP     7		/* ) */
-#define  T_START  8		/* ^ */
-#define  T_END    9		/* $ */
-#define  T_ANY   10		/* . */
-#define  T_CLASS 11		/* starts with [ */
-#define  T_SLASH 12		/*  \  */
-#define  T_CHAR  13		/* all the rest */
-#define  T_STR   14		/* string built of other tokens */
-#define  T_U     15
+typedef enum {
+    T_NONE = 0			/* no token */
+    ,T_OR			/* | */
+    ,T_CAT			/* binary operator */
+    ,T_STAR			/* * */
+    ,T_PLUS			/* + */
+    ,T_Q			/* ? */
+    ,T_LP			/* ( */
+    ,T_RP			/* ) */
+#ifdef NO_INTERVAL_EXPR
+#define T_LB	T_CHAR		/* { */
+#define T_RB	T_CHAR		/* } */
+#else
+    ,T_LB			/* { */
+    ,T_RB			/* } */
+#endif
+    ,T_START			/* ^ */
+    ,T_END			/* $ */
+    ,T_ANY			/* . */
+    ,T_CLASS			/* starts with [ */
+    ,T_SLASH			/*  \  */
+    ,T_CHAR			/* all the rest */
+    ,T_STR			/* string built of other tokens */
+    ,T_U
+} MAWK_TOKEN;
 
 /*  precedences and error codes  */
-#define  L   0
-#define  EQ  1
-#define  G   2
-#define  E1  (-1)
-#define  E2  (-2)
-#define  E3  (-3)
-#define  E4  (-4)
-#define  E5  (-5)
-#define  E6  (-6)
-#define  E7  (-7)
+#define  OP_L   0
+#define  OP_EQ  1
+#define  OP_G   2
+#define  ERR_1  (-1)
+#define  ERR_2  (-2)
+#define  ERR_3  (-3)
+#define  ERR_4  (-4)
+#define  ERR_5  (-5)
+#define  ERR_6  (-6)
+#define  ERR_7  (-7)
 
 #define  MEMORY_FAILURE      5
 
@@ -179,10 +162,16 @@ extern MACHINE RE_str(char *, size_t);
 extern MACHINE RE_class(BV *);
 extern void RE_cat(MACHINE *, MACHINE *);
 extern void RE_or(MACHINE *, MACHINE *);
-extern void RE_close(MACHINE *);
-extern void RE_poscl(MACHINE *);
+extern STATE *RE_close(MACHINE *);
+extern STATE *RE_poscl(MACHINE *);
 extern void RE_01(MACHINE *);
-extern void RE_panic(const char *) GCC_NORETURN;
+extern GCC_NORETURN void RE_panic(const char *, ...) GCC_PRINTFLIKE(1,2);
+
+#ifndef NO_INTERVAL_EXPR
+extern void RE_close_limit(MACHINE *, Int, Int);
+extern void RE_poscl_limit(MACHINE *, Int, Int);
+extern void duplicate_m(MACHINE *, MACHINE *);
+#endif
 
 #ifndef MAWK_H
 extern char *str_str(char *, size_t, char *, size_t);
@@ -203,47 +192,78 @@ extern RT_POS_ENTRY *RE_pos_stack_base;
 extern RT_POS_ENTRY *RE_pos_stack_limit;
 extern RT_POS_ENTRY *RE_pos_stack_empty;
 
-#ifdef LOCAL_REGEXP
+extern Int intrvalmin;
+extern Int intrvalmax;
+extern char *re_exp;
+
+#if defined(LOCAL_REGEXP) && defined(REGEXP_INTERNALS)
 static /* inline */ RT_POS_ENTRY *
 RE_pos_push(RT_POS_ENTRY * head, const RT_STATE * owner, const char *s)
 {
     head->pos = s;
     head->owner = (int) (owner - RE_run_stack_base);
 
-    if (++head == RE_pos_stack_limit)
+    if (++head == RE_pos_stack_limit) {
 	head = RE_new_pos_stack();
+    }
     head->prev_offset = 1;
     return head;
 }
 
-#if 0
-static /* inline */ const char *
-RE_pos_peek(const RT_POS_ENTRY * head)
-{
-    const RT_POS_ENTRY *prev = head - head->prev_offset;
-
-    /* peeking below the bottom node can be useful when debugging,
-     * so we allow it.  See RE_pos_stack_init().
-     */
-    return prev->pos;
-}
-#endif
-
 static /* inline */ const char *
 RE_pos_pop(RT_POS_ENTRY ** head, const RT_STATE * current)
 {
     RT_POS_ENTRY *prev = *head - (*head)->prev_offset;
 
-    if (prev->owner == current - RE_run_stack_base)	/* likely */
+    if (prev->owner == current - RE_run_stack_base) {	/* likely */
 	/* no need to preserve intervening nodes */
 	*head = prev;
-    else if (*head == prev)
+    } else if (*head == prev) {
 	RE_panic("unbalanced M_SAVE_POS and M_2JC");
-    else
+    } else {
 	(*head)->prev_offset += prev->prev_offset;
+    }
 
     return prev->pos;
 }
+
+#ifndef NO_INTERVAL_EXPR
+/* reset it_cnt to zero for the M_2JC state
+ *  which is where loop count is checked
+ */
+static void
+RE_init_it_cnt(STATE * s)
+{
+    STATE *p = s;
+    while (p->s_type < M_ACCEPT) {
+	if (p->s_type == M_2JC)
+	    p->it_cnt = 0;
+	p++;
+    }
+}
+#else
+#define RE_init_it_cnt(s)	/* nothing */
+#endif
+
+#ifndef NO_INTERVAL_EXPR
+#undef NO_RI_LOOP_UNROLL	/* experimental 2020/10/22 -TD */
+#ifdef NO_RI_LOOP_UNROLL
+#else
+static void
+RE_set_limit(STATE * s, Int minlimit, Int maxlimit)
+{
+    STATE *p = s;
+    while (p->s_type < M_ACCEPT) {
+	if (p->s_type == M_2JC) {
+	    p->it_min = minlimit;
+	    p->it_max = maxlimit;
+	}
+	p++;
+    }
+}
+#endif /* ! NO_RI_LOOP_UNROLL */
+#endif /* ! NO_INTERVAL_EXPR */
+
 #endif /* LOCAL_REGEXP */
 
 #endif /* REXP_H  */
